/*
 *
 *     Copyright (c) 2006, ClearSpeed Technology plc. All rights reserved.
 *
 * This material is provided "as is", with absolutely no warranty expressed or
 * implied. Any use is at your own risk. Permission to use or copy this software
 * for any purpose is hereby granted without fee, provided the above notices are
 * retained on all copies. Permission to modify the code and to distribute
 * modified code is granted, provided the above notices are retained, and a
 * notice that the code was modified is included with the above copyright notice.
 *
 */


/*!
  \file   cs_dft_demo_1d_c_fwd_rev.c
  \brief  Clearspeed DFT Library Test

  \date   30-January-2006

  This test demonstrates the creation of single precision 1d complex plans
  and their execution. It cycles through sizes 128 to 8192 in powers of 2,
  creating random data and then allocating a memory descriptor to the data.
  It then creates 2 1d single precision complex plans. One does a forward fft,
  the other a backward fft.

  It executes one after the other, such that the resulting output should be
  almost identical to the original input.

*/


#include <math.h>
#include <stdlib.h>

#include "csdft.h"
#include "cs_complex.h"

/*
    Routines to help compare input and golden image
*/
unsigned int compare_value(float val, float gold_val)
{
    float diff;

    if ( fabsf(gold_val) < 0.001)
    {
        diff = fabsf(val-gold_val);
    } else
    {
        diff = fabsf((val-gold_val)/gold_val);
    }
    if (diff > 0.05)
    {
        return 0;
    } else
    {
        return 1;
    }
}


unsigned int compareFft(FloatComplex *result, FloatComplex *gold, unsigned int w, unsigned int h)
{
    unsigned int r,c;

    for (r = 0; r < h; r++)
        for (c = 0; c < w; c++)
        {
            if (!compare_value(CS_crealf(result[r*w+c]), CS_crealf(gold[r*w+c]))||
                !compare_value(CS_cimagf(result[r*w+c]), CS_cimagf(gold[r*w+c])))
            {
                printf("Failed fft comparison locn, column %d, row %d\n", c, r);
                return 0;
            }
        }
    return 1;
}

/*
    Single precision complex 1d routine for size n
*/
unsigned int cs_dft_forward_reverse(unsigned int n)
{
    unsigned int i = 0, j = 0;
    unsigned int num_ffts = 100;
    unsigned int passed = 1;

    float v,vi;

    FloatComplex *in = NULL, *out = NULL;

    CSDftMemoryDescriptor md_in;
    CSDftMemoryDescriptor md_out;
    CSDftStatus           status;
    CSDftPlan             forward, reverse;


    /* malloc local space for ffts */
    in     = malloc(sizeof(FloatComplex) * n *num_ffts);
    out    = malloc(sizeof(FloatComplex) * n *num_ffts);
    


    /* Generate some random input */
    srand(1);

    for (j = 0; j < num_ffts; j++)
    {
        for (i=0; i<n ; i++)
        {
            do {
                v  = (1.0*rand())/RAND_MAX;
                vi = (1.0*rand())/RAND_MAX;
            } while (v < 0.02 || vi < 0.02);

            CS_complex_assign(in[j*n+i], v, vi);
        }
    }


    /* Create the host memory descriptors that point to the pre-allocated in and out arrays */
    md_in  = CSDFT_system_to_memory_descriptor((void *)in,  (sizeof(FloatComplex) * n) * num_ffts);
    
    md_out = CSDFT_system_to_memory_descriptor((void *)out, (sizeof(FloatComplex) * n) * num_ffts);


    /*
     Create the plan that describes the DFTs, in this case an unscaled single
     precision complex forward transform and a corresponding scaled backward one
    */
    forward = CSDFT_create_plan_1d(CSDFT_FORWARD |CSDFT_NATURAL_ORDER|CSDFT_COMPLEX|CSDFT_SINGLE, n, num_ffts, 1.0   ) ;
    reverse = CSDFT_create_plan_1d(CSDFT_BACKWARD|CSDFT_NATURAL_ORDER|CSDFT_COMPLEX|CSDFT_SINGLE, n, num_ffts, 1.0/n ) ;

    /* Perform the DFTs by calling the synchronous interface */
    status = CSDFT_execute_dft(forward, md_in, md_out) ;
    status = CSDFT_execute_dft(reverse, md_out, md_out) ;
    
    /* Error check */
    if (CSDFT_get_status() != CSDFT_NO_ERROR)
    {
    
       /* An error has occurred when processing kernel. Tell user the problem: */
       printf("%s\n", CSDFT_return_error_message(CSDFT_get_status()));
 
       /* Fail the test */
       passed = 0;
       return passed;
    
    }


    /* Check that the input and output are almost the same */
    for (j = 0; j < num_ffts; j++)
    {
        if (!compareFft(&out[j*n], &in[j*n], n, 1))
        {
            printf("DFT %d failed for size %d\n", j+1, n);
            passed = 0;
            break;
        }
    }


    /* Free the memory */
    free(out);
    free(in);


    /* Free the memory descriptors */
    CSDFT_free(md_out);
    CSDFT_free(md_in);


    /* Free the plans */
    CSDFT_free_plan(forward);
    CSDFT_free_plan(reverse);


    return passed;
}

/*
    Main
*/
int main(int argc, char *argv[])
{
    unsigned int s;
    unsigned int passed = 1;

    for (s = 128; s <= 8192; s = s*2)
    {
        if (!cs_dft_forward_reverse(s))
        {
            printf("Failed 1D single precision complex forward reverse DFT of size %d\n\n", s);
            passed  = 0;
        }
    }

    printf("\n**** 1D complex single precision forward reverse test: ");

    if (passed)
    {
        printf("passed\n");
    } else
    {
        printf("failed\n");
        return 1;
    }
    printf("\n");

    return 0;
}

