/*
 *
 *     Copyright (c) 2006, ClearSpeed Technology plc. All rights reserved.
 *
 * This material is provided "as is", with absolutely no warranty expressed or
 * implied. Any use is at your own risk. Permission to use or copy this software
 * for any purpose is hereby granted without fee, provided the above notices are
 * retained on all copies. Permission to modify the code and to distribute
 * modified code is granted, provided the above notices are retained, and a
 * notice that the code was modified is included with the above copyright notice.
 *
 */


/*!
  \file   cs_dft_demo_1d_c_perf.c
  \brief  Clearspeed DFT Library Test

  \date   06-December-2006

  This test demonstrates the creation of single precision 1d complex plans
  and their execution. It executes 1000 forward 1D FFTs of size 1024 in a 
  row, and then 1000 backward 1D FFTs of size 1024.  The result is then 
  compared with the input, and the number of FFTs per second is outputted.
  Data is left on the board in between FFTs to maximise performance.
  
  This test only uses one chip - to access both chips a user function should
  be used (this requires the SDK).


*/


#include <math.h>
#include <stdlib.h>

#include "csdft.h"
#include "cs_complex.h"
#include "timing.h"

/*
    Routines to help compare input and golden image
*/
unsigned int compare_value(float val, float gold_val)
{
    float diff;

    if ( fabsf(gold_val) < 0.001)
    {
        diff = fabsf(val-gold_val);
    } else
    {
        diff = fabsf((val-gold_val)/gold_val);
    }
    if (diff > 0.05)
    {
        return 0;
    } else
    {
        return 1;
    }
}


unsigned int compareFft(FloatComplex *result, FloatComplex *gold, unsigned int w, unsigned int h)
{
    unsigned int r,c;

    for (r = 0; r < h; r++)
        for (c = 0; c < w; c++)
        {
            if (!compare_value(CS_crealf(result[r*w+c]), CS_crealf(gold[r*w+c]))||
                !compare_value(CS_cimagf(result[r*w+c]), CS_cimagf(gold[r*w+c])))
            {
                printf("Failed fft comparison locn, column %d, row %d\n", c, r);
                return 0;
            }
        }
    return 1;
}

/*
    Single precision complex 1d routine for size 1024
*/
unsigned int cs_dft_performance()
{
    unsigned int i = 0, j = 0;
    unsigned int n = 1024;
    unsigned int num_ffts = 1000;
    unsigned int passed = 1;
    unsigned int symbol_address = 0;
    UniTime *time_start = NULL;
    UniTime *time_end = NULL;
    double milli_time = 0;

    float v,vi;

    FloatComplex *in = NULL, *out = NULL;

    CSDftMemoryDescriptor md_in;
    CSDftMemoryDescriptor md_out;
    CSDftMemoryDescriptor md_board;
    CSDftStatus           status;
    CSDftPlan             forward, reverse;
    
	time_start = malloc(sizeof(UniTime));
	time_end = malloc(sizeof(UniTime));

    /* malloc local space for ffts */
    in     = malloc(sizeof(FloatComplex) * n *num_ffts);
    out    = malloc(sizeof(FloatComplex) * n *num_ffts);
    


    /* Generate some random input */
    srand(1);

    for (j = 0; j < num_ffts; j++)
    {
        for (i=0; i<n ; i++)
        {
            do {
                v  = (1.0*rand())/RAND_MAX;
                vi = (1.0*rand())/RAND_MAX;
            } while (v < 0.02 || vi < 0.02);

            CS_complex_assign(in[j*n+i], v, vi);
        }
    }


    /* Create the host memory descriptors that point to the pre-allocated in and out arrays */
    md_in  = CSDFT_system_to_memory_descriptor((void *)in,  (sizeof(FloatComplex) * n) * num_ffts);
    
    md_out = CSDFT_system_to_memory_descriptor((void *)out, (sizeof(FloatComplex) * n) * num_ffts);
	
	/* Create board descriptor that will hold the data in between FFTs */
	
	/*
	First get symbol value --- "_sample_data" is predefined space on board usually
	internally used by the library when moving data from the host to the board.
	Since all our data is on the board, we can use this space.
	*/
	status = CSDFT_get_symbol_value("_sample_data", &symbol_address, 0,0);
	
	if(status!= CSDFT_NO_ERROR)
	{
	//error has occurred
	printf("An error has occurred: %s\n\n", CSDFT_return_error_message(status));
	return 0;
	} 
	
	//Now create the board memory descriptor
	md_board = CSDFT_processor_to_memory_descriptor(symbol_address, (sizeof(FloatComplex) * n) * num_ffts, 0, 0);
	
    /*
     Create the plan that describes the DFTs, in this case an unscaled single
     precision complex forward transform and a corresponding scaled backward one
    */
    forward = CSDFT_create_plan_1d(CSDFT_FORWARD |CSDFT_OPTIMAL_ORDER|CSDFT_COMPLEX|CSDFT_SINGLE, n, num_ffts, 1.0   ) ;
    reverse = CSDFT_create_plan_1d(CSDFT_BACKWARD|CSDFT_OPTIMAL_ORDER|CSDFT_COMPLEX|CSDFT_SINGLE, n, num_ffts, 1.0/n ) ;

    /* Dry run to pre-load twiddle factors for 1K size */
    status = CSDFT_execute_dft(forward, md_board, md_board) ;

	/* Copy initial data to board */
	CSDFT_memory_descriptor_copy(md_in, md_board, (sizeof(FloatComplex) * n) * num_ffts);
    
    /* Get time before FFTs */
    utGetTimeRaw( time_start );
    
    /* Execute forward FFT */
    status = CSDFT_execute_dft(forward, md_board, md_board) ;
        
    /* Get time after FFTs */
    utGetTimeRaw( time_end );
   
    /* Tell user how long test took and how many FFTs done/sec */
    milli_time = utGetMicrosecondsDiff(time_start, time_end);
    milli_time = milli_time/1000000;
    printf("Total time for forward FFTs: %f\nFFTs/sec: %f\n\n",milli_time, (num_ffts/milli_time) );

    /* Get time before FFTs */
    utGetTimeRaw( time_start );
    
    /* Execute backward FFT */
    status = CSDFT_execute_dft(reverse, md_board, md_board) ;
        
    /* Get time after FFTs */
    utGetTimeRaw( time_end );

    /* Tell user how long test took and how many FFTs done/sec */
    milli_time = utGetMicrosecondsDiff(time_start, time_end);
    milli_time = milli_time/1000000;
    printf("Total time for backward FFTs: %f\nFFTs/sec: %f\n\n",milli_time, (num_ffts/milli_time) );

    /* Error check */
    if (CSDFT_get_status() != CSDFT_NO_ERROR)
    {
    
       /* An error has occurred when processing kernel. Tell user the problem: */
       printf("%s\n", CSDFT_return_error_message(CSDFT_get_status()));
 
       /* Fail the test */
       passed = 0;
       return passed;
    
    }
    
    /* Retrieve results */
    CSDFT_memory_descriptor_copy(md_board, md_out, (sizeof(FloatComplex) * n) * num_ffts);

    /* Check that the input and output are almost the same  */
    for (j = 0; j < num_ffts; j++)
    {
        if (!compareFft(&out[j*n], &in[j*n], n, 1))
        {
            printf("DFT %d failed for size %d\n", j+1, n);
            passed = 0;
            break;
        }
    }
	

    /* Free the memory */
    free(out);
    free(in);
    free(time_start);
    free(time_end);


    /* Free the memory descriptors */
    CSDFT_free(md_out);
    CSDFT_free(md_in);
    CSDFT_free(md_board);


    /* Free the plans */
    CSDFT_free_plan(forward);
    CSDFT_free_plan(reverse);


    return passed;
}

/*
    Main
*/
int main(int argc, char *argv[])
{
    unsigned int passed = 1;

    
        if (!cs_dft_performance())
        {
            printf("Failed 1D complex single precision 1 chip performance test (1000 forward, 1000 backward board->board) \n\n");
            passed  = 0;
        }
    

    printf("\n**** 1D complex single precision 1 chip performance test (1000 forward, 1000 backward board->board): ");

    if (passed)
    {
        printf("passed\n");
    } else
    {
        printf("failed\n");
        return 1;
    }
    printf("\n");

    return 0;
}

