/* ------------------------------------------------------------------------------------------------
 *
 *        Copyright (c) ClearSpeed Technology plc 2000,2002,2006
 *        All rights reserved.
 *  
 *   This material is provided "as is", with absolutely no warranty expressed
 *   or implied. Any use is at your own risk. Permission to use or copy this
 *   software for any purpose is hereby granted without fee, provided the
 *   above notices are retained on all copies. Permission to modify the code
 *   and to distribute modified code is granted, provided the above notices
 *   are retained, and a notice that the code was modified is included with
 *   the above copyright notice.
*/
/* ------------------------------------------------------------------------------------------------
 *
 * Filename:	dgemm.c
 * ---------
 * 
 * Description:   Reports the computational rate, in Gflops per second,
 * ------------   of double-precision multiplication of two matrices.
 *
 *     This program creates simple matrices based on the dimensions input and
 *     calls dgemm_, described below, to perform matrix-matrix multiplication.
 *
 * Arguments:
 * ----------
 *   Name     Type     Dim    I/O     Description
 *   ----     ----     ---    ---     -----------
 *    m       int       -      I      Number of rows in matrix A.
 *    n	      int       -      I      Number of columns in matrix B.
 *    k	      int       -      I      Number of rows    in matrix B,
 *                                                      columns in matrix A.
 *    t	      int       -      I      Repeat calculations "t" times.
 *
 * Called Functions:
 * -----------------
 *   Name              Description
 *   ----              -----------
 *   gettimeofday      Gives the number of seconds and microseconds since the Epoch;
 *                     maintains these values in struct timeval.
 *
 *   dgemm_            Basic Linear Algebra Subroutine (BLAS), level 3 function that
 *                     performs one of the matrix-matrix operations:
 *
 *                     C := alpha*op( A )*op( B ) + beta*C
 *
 *                          where op( X ) = X 
 *                             or op( X ) = X_transpose
 *
 *                          alpha and beta are scalars, and A, B and C are matrices:
 *
 *                          op( A ) is m rows by k columns,
 *                          op( B ) is k rows by n columns, 
 *                          and C   is m rows by n columns.
 *
 * Notes:
 * ------
 *  1)  The library version of the DGEMM function is usually implemented in Fortran,
 *      and the end of the DGEMM argument list can vary, depending on the compiler.
 *      The call to dgemm_ used in this program requires two extra arguments at the
 *      end, both of type integer.  These extra arguments are the size of the character 
 *      strings transa and transb, respectively.  Arguments for dgemm_ are discussed
 *      in note 2.
 *
 *  2)  dgemm_ (transa, transb, m, n, k, alpha, A, lda, B, ldb, beta, C, ldc, lena, lenb)
 *
 *      Name     Type     Dim    I/O    Description
 *      ----     ----     ---    ---    -----------
 *      transa,  char      -     I      Indicates how to use the matrix stored in the array A (or B).
 *      transb                          Legal values are:
 *      
 *                                      'N' or 'n'  Use the matrix as it is.
 *                                      'T' or 't'  Use the transpose of the matrix.
 *                                      'C' or 'c'  Use the conjugate transpose of the matrix.
 *      
 *      m        int       -     I      Number of rows in matrix A.
 *      n        int       -     I      Number of columns in matrix B.
 *      k        int       -     I      Number of rows    in matrix B,
 *                                                columns in matrix A.
 *
 *      alpha    double    -     I      Scale factor for the input value of the matrix in A.
 *
 *      A        double  [m,k]   I      Two-dimensional array of size m-by-k for storing matrix A.
 *
 *      lda      int       -     I      Leading dimension of the array A.
 *                                       lda = max (1,k).  If transa = 'N' or 'n', lda = max (1,m).
 *
 *      B        double  [k,n]   I      Two-dimensional array of size k-by-n for storing matrix B.
 *
 *      ldb      int       -     I      Leading dimension of the array B.
 *                                       ldb = max(1,n).  If transb = 'N' or 'n', ldb = max (1,k).
 *
 *      beta     double    -     I      Scale factor for the input value of the matrix stored in C.
 *      
 *      C        double  [m,n]   I,O    Two-dimensional array of size m-by-n;
 *                                       on output, the result of alpha*op( A )*op( B ) + beta*C
 *      
 *      ldc      int       -     I      Leading dimension of the array C.
 *                                       ldc = max (1,m). 
 *
 *      lena     int       -     I      Length of character string in first argument, transa.
 *
 *      lenb     int       -     I      Length of character string in second argument, transb.
 *
 * Development History:
 * --------------------
 *    Date     Initials   Description
 *  02-May-06    RGL      Initial release.
 *
 ---------------------------------------------------------------------------------------------------
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

# include <sys/time.h>
typedef struct timeval UniTime;


/* Function prototypes */
void dgemm_(char *,char *,int *,int *,int *,double *,double *,int *,double *,int *,double *,double *,int *,int,int);

/* Structure for watching the clock */
UniTime tbgn, tend ;

/* Macros */
#define DEBUG 0


/**
 * Get the current time
 *
 * @param ret - pointer to UniTime type, where timing result is to be stored
 * @return 0 for success, !=0 for failure.
 */
int utGetTimeRaw( UniTime* ret )
{
    return gettimeofday(ret, NULL);
}


/**
 * Return difference between two timer values in microseconds.
 *
 * @param start Start time we want to compare against
 * @param end End time we want to compare against
 *
 * @return difference between start and end times in microseconds
 */
int utGetMicroseconds( UniTime* start , UniTime* end )
{
    return (int)(end->tv_sec - start->tv_sec)*1000000 + (int)(end->tv_usec - start->tv_usec);
}


int main(int argc, char *argv[]) 
{
   /* Declare variables and set defaults */
   double *a, *A, *b, *B, *c, *C;
   double  alpha = (double)1.0 ;
   double  beta  = (double)1.0;
   int     m=1152, n=1152, k=1152, i;
   int     t, ntimes=1;
   int     err=-1;
   char    trans[2]="N";
   double  seconds;
   int     minutes;
   double  expected, flops;
   double  sec_per_usec=(double)1.e-06;
   double  Gflop_per_flop=(double)1.e-09;

   /* Process the command-line */
   int   arg=1;
   char *argErr="";

   while (arg<argc)
   {
       if (!strcmp(argv[arg], "-h") || !strcmp(argv[arg], "--help"))
       {
           printf("\nUsage: dgemm_example [option]\n\n") ;
           printf("Following options may be given:\n\n") ;
           printf("-m    Matrix A has 'm' rows. Default is 1152.\n") ;
           printf("-n    Matrix B has 'n' columns. Default is 1152.\n") ;
           printf("-k    Matrix A has 'k' columns. Default is 1152.\n") ;
           printf("      Matrix B has 'k' rows\n") ;
           printf("-t    Repeat calculations 't' times. Defaults to 1.\n") ;
           printf("-h    display this help and exit\n\n") ;
           
           return -1 ;
       }
       
       if (!strcmp(argv[arg], "-m"))
       {
           if (++arg<argc)
           {
               m = atoi(argv[arg++]);
           } 
           else 
           {
               argErr="m";
           }
       } 
       else if (!strcmp(argv[arg], "-n"))
       {
           if (++arg<argc)
           {
               n = atoi(argv[arg++]);
           } 
           else 
           {
               argErr="n";
           }
       } 
       else if (!strcmp(argv[arg], "-k")) 
       {
           if (++arg<argc) 
           {
               k = atoi(argv[arg++]);
           } 
           else 
           {
               argErr="k";
           }
       } 
       else if (!strcmp(argv[arg], "-t")) 
       {
           if (++arg<argc) 
           {
               ntimes = atoi(argv[arg++]);
           } 
           else 
           {
               argErr="t";
           }
       } 
       else 
       {
           printf("Unrecognized input: %s\n", argv[arg]);
           printf("Try 'dgemm_example -h' for more information.\n\n") ;
           
           return(-1);
       }

       if (strcmp(argErr, "")) 
       {
           printf("Error:  Missing value for argument '-%s'\n", argErr);
           return(-1);
       }
   }

   /* Allocate memory for the matrices */
   /* -------------------------------- */
   A = (double *)malloc(m*k*sizeof(double));
   if ( A == NULL ) 
   {
       printf( "Not enough memory available\n" );
       exit(0);
   }
   
   B = (double *)malloc(k*n*sizeof(double));
   if ( B == NULL ) 
   {
       printf( "Not enough memory available\n" );
       free(B);
       exit(0);
   }
   
   C = (double *)malloc(m*n*sizeof(double));
   if ( C == NULL ) 
   {
       printf( "Not enough memory available\n" );
       free(B);
       free(A);
       exit(0);
   }

   for (i=0;i<m*k;i++) 
   {                           /* Initialize matrix A */
       A[i] = (double)1.0;
   }
   for (i=0;i<k*n;i++) 
   {                           /* Initialize matrix B */
       B[i] = (double)2.0;
   }

   /* Repeat calculations ntimes */
   /* -------------------------- */
   for (t=0;t<ntimes;t++) 
   {
       err = -1;

       for (i=0;i<m*n;i++)
       {                        /* (Re)initialize matrix C */
           C[i] = (double)3.0;
       }
       /* Expected result in each matrix element */
       expected = (double)(k*alpha*(*A)*(*B) + beta*(*C));

       if ( utGetTimeRaw(&tbgn) )      /* Start the wall clock */
       {
           printf("*** Error *** dgemm.c:  utGetTimeRaw() failed before call to dgemm_().\n");
           continue;
       } 

       /* ----------------------------------------------------------------- */
       /*  Multiply matrix A by matrix B such that C := alpha*A*B + beta*C  */
       /* ----------------------------------------------------------------- */
       dgemm_(trans, trans, &m, &n, &k, &alpha, A, &m, B, &k, &beta, C, &m, 1, 1);

       if ( utGetTimeRaw(&tend) )      /* Stop the wall clock */
       {
           printf("*** Error *** dgemm_c:  utGetTimeRaw() failed after call to dgemm_().\n");
           continue;
       }

       seconds = utGetMicroseconds(&tbgn, &tend)*sec_per_usec ;                      /* Time elapsed for dgemm */

       for (i=0 ; i<m*n ; i++) 
       {                        /* Check the answer */
           if (C[i] != expected) 
           {
               err = i;
               break;
           }
       }

       /* ------------------------------- */
       /*  Report results                 */
       /* ------------------------------- */
       printf("Iteration %d\t",t);

       if (err == -1) 
       {
           flops = (double)(2*m) * (double)(n) * (double)(k);
           printf("C(%d, %d),k=%d\t",m,n,k);
           minutes = seconds / 60;
           //printf("Time: %e sec\t",seconds);
           printf("Time: %d min %6.3f sec\t",minutes,(seconds-(minutes*60)));
#if DEBUG
           printf("Rate: %f GF / %f sec = %f GF/s\n", (flops * Gflop_per_flop),seconds, (flops * Gflop_per_flop)/seconds);
#else
           printf("GF/s: %f\n", (flops * Gflop_per_flop)/seconds);
#endif
       } 
       else 
       {
           printf("*** Error *** At element %d expected %e, but obtained %e\n", err, expected, C[err]);
           continue;
       }
   }

   printf("Finished.\n");

   /* ------------------------------- */
   /*  Release memory                 */
   /* ------------------------------- */
   free(A);
   free(B);
   free(C);

   return(0);
}
