//
//
//      Copyright (c) ClearSpeed Technology plc 1997-2006
//      All rights reserved.
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. Any use is at your own risk. Permission to use or copy this
// software for any purpose is hereby granted without fee, provided the
// above notices are retained on all copies. Permission to modify the code
// and to distribute modified code is granted, provided the above notices
// are retained, and a notice that the code was modified is included with
// the above copyright notice.
//
//


#ifndef _CLEARD_INTERFACE_CSAPI_INCLUDE_CSAPI_H_
#define _CLEARD_INTERFACE_CSAPI_INCLUDE_CSAPI_H_

#include "csapi_errno.h"
#include <stdarg.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Version numbers for this header file. The major version must equal that expected as it will change when function names or
   parameters are changed. The minor version should be greater than or equal to that expected as it will be incremented when
   new functions or definitions are added and when comments are modified. */
#define CSAPI_HEADER_VERSION_MAJOR 1
#define CSAPI_HEADER_VERSION_MINOR 1

/* Processor index definitions for proc_inx parameter */
#define CS_PROCESSOR0    0
#define CS_PROCESSOR1    1

/* Endianness definitions for CSAPI_endianness */
#define CS_LITTLE_ENDIAN 0
#define CS_BIG_ENDIAN    1

/* Instance definition for use with CSAPI_connect function */
#define CSAPI_INSTANCE_ANY ((unsigned)-1)

/* Reset flags for use with CSAPI_reset and CSAPI_set_system_param functions */
#define CSAPIFlags_FULL_SYSTEM_RESET 1 /* CCBR, CCIs, DDRs and Bus monitor, plus usual Processor reset */

/* Enumerations for use with CSAPI_set_system_param function */
enum CSAPISystemParameters
{ 
    CSP_Verbosity=0         , /* Set with integer value (0 or 1) */
    CSP_ResetFlags          , /* Set with integer value */
    CSP_LogLevel            , /* Set with integer value */
    CSP_ForceBusBigEndian   , /* Set with integer value (0 or 1 only) */
    CSP_ForceBusLittleEndian, /* Set with integer value (0 or 1 only) */
    CSP_NoZeroBss           , /* Set with integer value (0 or 1) */
    CSP_LoadFuseMask        , /* Set with string value (filename) */
    CSP_SaveFuseMask        , /* Set with string value (filename) */
    CSP_FuseStrictCheck     , /* Set with integer value (0 or 1) */
    CSP_NoFPGAModel         , /* Set with integer value (0 or 1) */
    CSP_CCBRClockSlow       , /* Set with integer value (75, 80, <100>, 250, 300) */
    CSP_CCBRClockFast       , /* Set with integer value (100, <200>, 250, 300, 375, 400) */
    CSP_DDRClock            , /* Set with integer value (133, 166, <200>, 233, 266) */
    CSP_IcacheBypassActive  , /* Set with integer value (0 or 1) */
    CSP_DcacheBypassActive  , /* Set with integer value (0 or 1) */
    CSP_DdrEccOn            , /* Set with integer value (0 or 1) */
    CSP_TraceMode             /* Used internally by debug / trace layer */
};

/* Enumerations for use with CSAPI_new function */
enum CSAPIMode
{
    CM_NotLoaded=0,
    CM_Direct ,
    CM_Trace,
    CM_Debugger
};

/* Enumerations for use with the CSAPI_version function */
enum CSAPIVersionType
{
    CSV_CsapiHeader,               /* (major + minor) CSAPI header versions defined above */
    CSV_RuntimePackage,            /* (major + minor) Version of the installed runtime package */
    CSV_RuntimeBuild,              /* (string)        Internal build version of the runtime */
    CSV_Fpga,                      /* (major)         Version of the FPGA on the connected board */
    CSV_Mtap                       /* (major)         Version of the first MTAP on the connected board */
};

struct CSAPIState;
struct DRVEvent;
struct DRVProcess;

#define CS_MAX_VERSION_STRING_LENGTH 80
struct CSAPIVersion
{
    unsigned int major;
    unsigned int minor;
    char string[CS_MAX_VERSION_STRING_LENGTH];
};

typedef void (*CSAPI_EventFnPtr)(struct CSAPIState *, struct DRVEvent *, void *);

/* Description: Loads the dynamic library for the selected mode and creates a new instance of the API state.
                A pointer to the API state structure is passed as a parameter to all of the CSAPI functions.
   Returns: A pointer to a newly created and initialized instance of the API state structure. */
struct CSAPIState* CSAPI_new(
    enum CSAPIMode mode );         /* Use CM_Direct unless attaching the debugger or using tracing */

/* Description: Unloads the dynamic library and destroys the instance of the API state. It is not possible
                to call any CSAPI function with this state once it has been passed to CSAPI_delete.
   Returns: Nothing. */
void CSAPI_delete(
    struct CSAPIState* const s );  /* State created by CSAPI_new. Will be destroyed by this function */

/* Description: Returns the version number of the object specified by the 'ver_type' parameter. The result will use the relevant
                members of the CSAPIVersion structure (defined above), which includes major, minor and string.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_not_connected */
DRVErrno CSAPI_version(
    struct CSAPIState* const s,    /* State created by CSAPI_new. Required to ensure the dynamic CSAPI library is loaded */
    enum CSAPIVersionType ver_type, /* Object to return the version for, as defined in the CSAPIVersionType enumeration above */
    struct CSAPIVersion* version ); /* Pointer to value to be given the version. Members used for each type are defined above */

/* Description: Connects to the driver either locally or on the specified host. Connect should be called once
                for each CSAPIState created with CSAPI_new. Disconnect by deleting the state with CSAPI_delete.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_failed_to_create_thread, DRVErrno_lldclient_error,
            DRVErrno_fpga_upgrade_required, DRVErrno_error */
DRVErrno CSAPI_connect(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    const char* host_or_addr,      /* Host on which the simulator is running or NULL for local hardware.
                                      For host name use either localhost, DNS name or IP address N.N.N.N. */
    unsigned int instance );       /* Instance number for hardware or simulator, starting from zero.
                                      Use CSAPI_INSTANCE_ANY to connect to any hardware available */

/* Description: Resets the processor specified by proc_inx. This includes resetting the DMA, GSU and PIO engine, stopping
                and setting up the TSC and setting up the system endianness and instruction cache. The hardware semaphores
                and interrupts are then initialised and the bootstrap code is run to set up the microcode.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_not_connected, DRVErrno_error */
DRVErrno CSAPI_reset(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to be reset. CS_PROCESSOR0 or CS_PROCESSOR1 */
    unsigned int flags );          /* Use CSAPIFlags_FULL_SYSTEM_RESET to reset CCBR, CCIs, DDRs and Bus monitor */

/* Description: Each connecting application needs to register itself with the driver. This tells the driver that the
                processor is active and tells the processor where to send its events.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_not_connected,
            DRVErrno_kernel_max_app_count_exceeded */
DRVErrno CSAPI_register_application(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx );       /* Index of processor to register application for. CS_PROCESSOR0 or CS_PROCESSOR1 */

/* Description: Loads the executable code from a .csx file onto a specified processor.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_bad_csapi_arg,
            DRVErrno_not_connected, DRVErrno_cannot_open_file, DRVErrno_error */
DRVErrno CSAPI_load(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to load application to. CS_PROCESSOR0 or CS_PROCESSOR1 */
    const char* prog_name );       /* Specifies the path to the CSX executable, which is relative to the CSPATH environment
                                      variable. If the program is in the local directory, ./ must be used or be in CSPATH */

/* Description: Sets process to contain the pointer to the previously loaded executable.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_bad_csapi_arg,
            DRVErrno_not_connected, DRVErrno_connection_broken */
DRVErrno CSAPI_get_last_loaded_handle(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to get last process pointer for. CS_PROCESSOR0 or CS_PROCESSOR1 */
    struct DRVProcess** process ); /* Pointer to value to be given a pointer to the last loaded process on the processor */

/* Description: Unloads the specified process from the specified processor.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_bad_csapi_arg, DRVErrno_error */
DRVErrno CSAPI_unload(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to unload application from. CS_PROCESSOR0 or CS_PROCESSOR1 */
    struct DRVProcess* process );  /* Pointer to the process to be unloaded */

/* Description: Executes a program loaded onto the given processor.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_not_connected, DRVErrno_error */
DRVErrno CSAPI_run(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx );       /* Index of processor to start (continue) running. CS_PROCESSOR0 or CS_PROCESSOR1 */

/* Description: Executes the specified process on the processor it has been loaded on
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_bad_csapi_arg, 
            DRVErrno_not_connected, DRVErrno_error */
DRVErrno CSAPI_run_process(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to start (continue) running. CS_PROCESSOR0 or CS_PROCESSOR1 */
    struct DRVProcess* process );  /* Pointer to the process to execute */

/* Description: Halts the execution of a program running on the specified processor.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_not_connected,
            DRVErrno_permission_denied, DRVErrno_error */
DRVErrno CSAPI_halt(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx );       /* Index of processor to stop running. CS_PROCESSOR0 or CS_PROCESSOR1 */

/* Description: Restarts the execution of a halted program.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_not_connected, DRVErrno_error */
DRVErrno CSAPI_start(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx );       /* Index of processor to start (restart) running. CS_PROCESSOR0 or CS_PROCESSOR1 */

/* Description: Awaits the termination signal from the program running on the given processor.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx */
DRVErrno CSAPI_wait_on_terminate(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx );       /* Index of processor to wait on for termination. CS_PROCESSOR0 or CS_PROCESSOR1 */

/* Description: Obtains the return status of the CSX application which has just terminated. This should be called after
                CSAPI_wait_on_terminate. If the function is not successful, return_value is undefined.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_bad_csapi_arg */
DRVErrno CSAPI_get_return_value(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to get return value from. CS_PROCESSOR0 or CS_PROCESSOR1 */
    int* const return_value );     /* Pointer to value to be given the return value from the terminated CSX program */

/* Description: Writes the given value to a control register at address reg_addr in the processor specified by proc_inx.
                Use of this function requires detailed knowledge of the architecture and the function of the control registers.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_not_connected, DRVErrno_error */
DRVErrno CSAPI_write_control_register(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to write control register on. CS_PROCESSOR0 or CS_PROCESSOR1 */
    unsigned int reg_addr,         /* Address of control register to write to */
    unsigned int value );          /* New value to be written to the control register */

/* Description: Same as CSAPI_write_control_register, except that the reg_addr must be shifted for the appropriate
                processor index before calling this function. This allows for a more efficient implementation.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_not_connected, DRVErrno_error */
DRVErrno CSAPI_write_control_register_raw(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int reg_addr,         /* Address of control register to write to */
    unsigned int value );          /* New value to be written to the control register */

/* Description: Reads the control register at address reg_addr from the processor specified by proc_inx into the memory
                location pointed to by value. If the function is not successful, the value is undefined.
                Use of this function requires detailed knowledge of the architecture and the function of the control registers.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_bad_csapi_arg,
            DRVErrno_not_connected, DRVErrno_error */
DRVErrno CSAPI_read_control_register(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to read control register on. CS_PROCESSOR0 or CS_PROCESSOR1 */
    unsigned int reg_addr,         /* Address of control register to read from */
    unsigned int* const value );   /* Pointer to memory to be given the value read from the control register */

/* Description: Same as CSAPI_read_control_register, except that the reg_addr must be shifted for the appropriate
                processor index before calling this function. This allows for a more efficient implementation.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_bad_csapi_arg, DRVErrno_not_connected, DRVErrno_error */
DRVErrno CSAPI_read_control_register_raw(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int reg_addr,         /* Address of control register to read from */
    unsigned int* const value );   /* Pointer to memory to be given the value read from the control register */

/* Description: Copies data from a buffer on the host pointed to by data_in, to a location in mono memory (DRAM) on the board
                given by address. The size is specified in bytes. The processor specified by proc_inx is halted and the data
                cache is flushed during the transfer. The appropriate processor for the mono memory address should be used.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_bad_csapi_arg,
            DRVErrno_not_connected, DRVErrno_error */
DRVErrno CSAPI_write_mono_memory(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to halt while writing to DRAM memory. CS_PROCESSOR0 or CS_PROCESSOR1 */
    unsigned int address,          /* Address of mono memory (DRAM on board) to start writing to */
    unsigned int size,             /* Number of bytes to copy from data_in (on host) to address (on board) */
    const void* data_in );         /* Address on host to start copying data from */

/* Description: Same as CSAPI_write_mono_memory, except that the processor is not halted and the data cache is not flushed.
                This may cause the transfer to take longer, but allows the processor to continue during the transfer.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_bad_csapi_arg, DRVErrno_not_connected, DRVErrno_error */
DRVErrno CSAPI_write_mono_memory_raw(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int address,          /* Address of mono memory (DRAM on board) to start writing to */
    unsigned int size,             /* Number of bytes to copy from data_in (on host) to address (on board) */
    const void* data_in );         /* Address on host to start copying data from */

/* Description: Copies data from a location in mono memory (DRAM) on the board given by address, to a pre-allocated buffer on
                the host pointed to by data_out. The size is specified in bytes. The processor specified by proc_inx is halted
                and the data cache is flushed during the transfer. The appropriate processor for the mono memory address should
                be used. The data_out buffer must be allocated on the host before calling this function.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_bad_csapi_arg,
            DRVErrno_not_connected, DRVErrno_error */
DRVErrno CSAPI_read_mono_memory(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to halt while reading from DRAM memory. CS_PROCESSOR0 or CS_PROCESSOR1 */  
    unsigned int address,          /* Address of mono memory (DRAM on board) to start reading from */
    unsigned int size,             /* Number of bytes to copy from address (on board) to data_out (on host) */
    void* const data_out );        /* Pre-allocated address on host to start copying data to */

/* Description: Same as CSAPI_read_mono_memory, except that the processor is not halted and the data cache is not flushed.
                This may cause the transfer to take longer, but allows the processor to continue during the transfer.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_bad_csapi_arg, DRVErrno_not_connected, DRVErrno_error */
DRVErrno CSAPI_read_mono_memory_raw(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int address,          /* Address of mono memory (DRAM on board) to start reading from */
    unsigned int size,             /* Number of bytes to copy from address (on board) to data_out (on host) */
    void* const data_out );        /* Pre-allocated address on host to start copying data to */

/* Description: Starts the CSAPI_write_mono_memory function in a separate thread and returns immediately. The host is
                then free to continue in the current thread. The host can check when the transfer has completed by using the
                CSAPI_write_mono_memory_async_poll function, or it can wait for the transfer to complete by using the blocking
                CSAPI_write_mono_memory_async_wait function.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_bad_csapi_arg,
            DRVErrno_failed_to_signal_semaphore, DRVErrno_failed_while_waiting_for_semaphore */
DRVErrno CSAPI_write_mono_memory_async(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to halt while writing to DRAM memory. CS_PROCESSOR0 or CS_PROCESSOR1 */
    unsigned int address,          /* Address of mono memory (DRAM on board) to start writing to */
    unsigned int size,             /* Number of bytes to copy from data_in (on host) to address (on board) */
    const void* data_in );         /* Address on host to start copying data from */

/* Description: Allows the host to wait for the completion of an asynchronous transfer started by CSAPI_write_mono_memory_async.
                The wait will only return once the transfer has completed or if there is an error. The return status should be
                checked to ensure that the transfer completed successfully.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_failed_while_waiting_for_semaphore,
            DRVErrno_failed_to_signal_semaphore */
DRVErrno CSAPI_write_mono_memory_async_wait(
    struct CSAPIState* const s );  /* State created by CSAPI_new */

/* Description: Allows the host to check the status of an asynchronous transfer started by CSAPI_write_mono_memory_async.
                The status is passed back using the completed parameter. The return status should be checked to ensure that a
                valid status was obtained.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_bad_csapi_arg, DRVErrno_failed_while_waiting_for_semaphore */
DRVErrno CSAPI_write_mono_memory_async_poll(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int* const completed ); /* Pointer to value to be given the status of the transfer in progress */

/* Description: Starts the CSAPI_read_mono_memory function in a separate thread and returns immediately. The host is
                then free to continue in the current thread. The host can check when the transfer has completed by using the
                CSAPI_read_mono_memory_async_poll function, or it can wait for the transfer to complete by using the blocking
                CSAPI_read_mono_memory_async_wait function.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_bad_csapi_arg,
            DRVErrno_failed_to_signal_semaphore, DRVErrno_failed_while_waiting_for_semaphore */
DRVErrno CSAPI_read_mono_memory_async(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to halt while reading from DRAM memory. CS_PROCESSOR0 or CS_PROCESSOR1 */
    unsigned int address,          /* Address of mono memory (DRAM on board) to start reading from */
    unsigned int size,             /* Number of bytes to copy from address (on board) to data_out (on host) */
    void* const data_out );        /* Pre-allocated address on host to start copying data to */

/* Description: Allows the host to wait for the completion of an asynchronous transfer started by CSAPI_read_mono_memory_async.
                The wait will only return once the transfer has completed or if there is an error. The return status should be
                checked to ensure that the transfer completed successfully.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_failed_while_waiting_for_semaphore,
            DRVErrno_failed_to_signal_semaphore */
DRVErrno CSAPI_read_mono_memory_async_wait(
    struct CSAPIState* const s );  /* State created by CSAPI_new */

/* Description: Allows the host to check the status of an asynchronous transfer started by CSAPI_read_mono_memory_async.
                The status is passed back using the completed parameter. The return status should be checked to ensure that a
                valid status was obtained.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_bad_csapi_arg, DRVErrno_failed_while_waiting_for_semaphore */
DRVErrno CSAPI_read_mono_memory_async_poll(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int* const completed ); /* Pointer to value to be given the status of the transfer in progress */

/* Description: Copies and converts data from the endianness used by the processor proc_inx in the current connection to the
                native host endianness. The number of bytes to copy and convert from in to out is set by the size parameter.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_bad_csapi_arg,
            DRVErrno_not_connected */
DRVErrno CSAPI_buffer_to_native_endian(
    struct CSAPIState* const s,    /* State created by CSAPI_new. Required to ensure the dynamic CSAPI library is loaded */
    unsigned int proc_inx,         /* Index of processor to convert endianness from. CS_PROCESSOR0 or CS_PROCESSOR1 */
    void* out,                     /* Pointer to memory to write out going data to, in native host endianness */
    const void* in,                /* Pointer to memory to read incoming data from, in the selected processor endianness */
    int size );                    /* Number of bytes to copy and convert */

/* Description: Gets the current endianness of the specified processor on the current board.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_bad_csapi_arg */
DRVErrno CSAPI_endianness(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor get endianness of. CS_PROCESSOR0 or CS_PROCESSOR1 */
    unsigned int* const endian );  /* Pointer to value to be given the current endianness of the specified processor on the
                                      current board. Value will be set to either CS_LITTLE_ENDIAN or CS_BIG_ENDIAN */

/* Description: Sets the specified system parameter (using the CSAPISystemParameters enumeration) to a value given to either
                the vi or vs parameter as appropriate. See the CSAPISystemParameters enumeration to see which parameters use
                which input variable. The user documentation should be referred to for valid combinations of system parameters.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_bad_csapi_arg, DRVErrno_error */
DRVErrno CSAPI_set_system_param(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    enum CSAPISystemParameters what, /* Name of parameter to be set, defined by an enumeration at the top of this file */
    unsigned int vi,               /* Integer value to set the parameter with (if applicable) */
    const char* vs );              /* String value to set the parameter with (if applicable) */

/* Description: Switches to the specified thread on specified processor, identified by thread ID. The previous thread ID is
                returned in old_thread_id so that we can switch back to it if necessary.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_bad_csapi_arg,
            DRVErrno_not_connected, DRVErrno_permission_denied, DRVErrno_error */
DRVErrno CSAPI_set_thread(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to set thread on. CS_PROCESSOR0 or CS_PROCESSOR1 */
    unsigned int thread_id,        /* ID of thread to switch to on processor */
    unsigned int* const old_thread_id ); /* Pointer to value to be given the ID of thread that was running on processor */

/* Description: Provides the total number of threads supported by the specified processor. Note this this will include
                threads used by the debugger or other system applications.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_bad_csapi_arg */
DRVErrno CSAPI_num_threads(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to count threads on. CS_PROCESSOR0 or CS_PROCESSOR1 */
    unsigned int* const num_of_threads ); /* Pointer to value to be given the number of threads supported by the processor */

/* Description: Registers that the host is intending to wait on the specified semaphore number on the specified processor.
                Registration must be performed only on semaphores that the host will be waiting on.
                A race condition will be created if the host signals, or the board processor waits on, a registered semaphore.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_not_connected, DRVErrno_error */
DRVErrno CSAPI_register_semaphore(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to register semaphore on. CS_PROCESSOR0 or CS_PROCESSOR1 */
    unsigned int sem_number );     /* Semphore number to register on processor */

/* Description: Allows the host to wait on the specified semaphore on the specified processor.
                The semaphore must be registered before it can be waited upon.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_failed_while_waiting_for_semaphore,
            DRVErrno_semaphore_number_out_of_bounds, DRVErrno_error */
DRVErrno CSAPI_semaphore_wait(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor wait for semaphore on. CS_PROCESSOR0 or CS_PROCESSOR1 */
    unsigned int sem_number );     /* Semaphore number to wait on */

/* Description: Signals the specified semaphore on the specified processor.
                The semaphore must be NOT be registered if it is going to be signalled by the host.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_not_connected,
            DRVErrno_semaphore_registered, DRVErrno_error */
DRVErrno CSAPI_semaphore_signal(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to signal semaphore on. CS_PROCESSOR0 or CS_PROCESSOR1 */
    unsigned int sem_number );     /* Semaphore number to signal */

/* Description: Provides the total number of semaphores supported by the specified processor. Note this this will include
                system semaphores, and semaphores used by the debugger and asynchronous memory read / write functions.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_bad_csapi_arg */
DRVErrno CSAPI_num_semaphores(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to count semaphore on. CS_PROCESSOR0 or CS_PROCESSOR1 */
    unsigned int* const num_of_semaphores ); /* Pointer to value to be given the number of semaphores supported */

/* Description: Gets the function call back registered with CSAPI_register_callback for the specified event.
                Typically used to check see if a call back is registered, or to chain user call back functions.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_bad_csapi_arg, DRVErrno_error */
DRVErrno CSAPI_get_callback(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int event,            /* Event to trigger the call back function */
    CSAPI_EventFnPtr* const event_cb_out ); /* Pointer to value to be given the call back Function pointer */

/* Description: Registers a user call back function for an event signalled by the board. When the event occurs the event
                call back function will be called with the state, event data and a pointer to specified user data.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_bad_csapi_arg, DRVErrno_error */
DRVErrno CSAPI_register_callback(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int event,            /* Event to trigger the call back function */
    CSAPI_EventFnPtr event_cb,     /* Function pointer to an even handler on the host. See definition at top of file */
    void* user_data );             /* Pointer to user data to be passed to the event handler on the host */

/* Description: Returns the total available mono memory (DRAM) local to the specified processor.
                Processors can see mono memory on other processors but will not access this as efficiently as local memory.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_bad_csapi_arg,
            DRVErrno_program_running */
DRVErrno CSAPI_get_free_mem(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to get free memory on. CS_PROCESSOR0 or CS_PROCESSOR1 */
    unsigned int mem_inx,          /* Index of memory. Currently unused as free space can only be used for mono memory */
    unsigned int* const size_out ); /* Pointer to value to be given the current amount of free memory local to the processor */

/* Description: Allocates mono memory (DRAM) local to the specified processor for use by both the CSX program and the host.
                The memory allocation can use an alignment and the allocated address will be given to the variable specified
                by 'symbol_name' in the current CSX program. The same address will be returned in the mem_ptr_out parameter.
                Processors can see mono memory on other processors but will not access this as efficiently as local memory.
                Memory allocation and releasing can only be done when the processor is not running.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_bad_csapi_arg, DRVErrno_program_running,
            DRVErrno_no_symbol, DRVErrno_cannot_open_file, DRVErrno_not_enough_blocks, DRVErrno_not_enough_memory, DRVErrno_error */
DRVErrno CSAPI_allocate_shared_memory(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to allocate shared memory on. CS_PROCESSOR0 or CS_PROCESSOR1 */
    unsigned int mem_inx,          /* Index of memory. Currently unused as only mono memory can be allocated */
    unsigned int size,             /* Number of contiguous bytes to attempt to allocate in the free space */
    unsigned int alignment,        /* Bytes to align allocated memory to on board */
    const char* symbol_name,       /* Name of static global variable in the CSX program to be given the allocated address */
    unsigned int* const mem_ptr_out ); /* Pointer to value to be given the address of the allocated memory on the processor */

/* Description: Attempts to allocate mono memory (DRAM) local to the specified processor at the address given by mem_ptr_in.
                This is the same as CSAPI_allocate_shared_memory, except that instead of returning an address for the allocated
                memory, it simply returns success or fail for allocating at the address given by mem_ptr_in. This can be useful
                for supporting programs where the addresses of the data arrays are hard coded in the CSX program.
                Processors can see mono memory on other processors but will not access this as efficiently as local memory.
                Memory allocation and releasing can only be done when the processor is not running.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_bad_csapi_arg, DRVErrno_program_running,
            DRVErrno_no_symbol, DRVErrno_cannot_open_file, DRVErrno_not_enough_blocks, DRVErrno_address_in_use, DRVErrno_error */
DRVErrno CSAPI_allocate_static_shared_memory(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to allocate static shared memory on. CS_PROCESSOR0 or CS_PROCESSOR1 */
    unsigned int mem_inx,          /* Index of memory. Currently unused as only mono memory can be allocated */
    unsigned int mem_ptr_in,       /* Desired address of the allocated memory on the processor */
    unsigned int size,             /* Number of bytes to attempt to allocate at the mem_ptr_in address */
    const char* symbol_name );     /* Name of static global variable in the CSX program to be given the allocated address */

/* Description: Releases previously allocated memory local to the specified processor at the address given by mem_ptr.
                This memory on the board is then available to be allocated by CSAPI_allocate_shared_memory.
                Processors can see mono memory on other processors but will not access this as efficiently as local memory.
                Memory allocation and releasing can only be done when the processor is not running.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_program_running,
            DRVErrno_failed_to_find_memory_block */
DRVErrno CSAPI_free(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to release allocated memory on. CS_PROCESSOR0 or CS_PROCESSOR1 */
    unsigned int mem_ptr );        /* Address of the allocated memory on the processor, which is to be released */

/* Description: Returns the value of the symbol symbol_name in the given CSX program. This is typically the address of the
                corresponding static variable in the program. The address can then be used with the CSAPI_read_mono_memory and
                CSAPI_write_mono_memory functions to access the contents of the memory on the board pointed to by the variable.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_bad_csapi_arg, DRVErrno_no_symbol, DRVErrno_cannot_open_file,
            DRVErrno_error */
DRVErrno CSAPI_get_symbol_value(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    const char* prog_name,         /* Name of the CSX program that declares the symbol */
    const char* symbol_name,       /* Name of the symbol to obtain the value for (typically a static variable) */
    unsigned int* const symbol_value_out ); /* On success this is set to the value associated with symbol_name.
                                               (typically the address of a static variable given by symbol_name) */

/* Description: Returns the symbol value as loaded in the given process. This will include any relocations that have been applied
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_bad_csapi_arg, DRVErrno_no_symbol */
DRVErrno CSAPI_get_symbol_value_loaded(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    struct DRVProcess* process,    /* Pointer to the process containing the given symbol */
    const char* symbol_name,       /* Name of the symbol to obtain the value for (typically a static variable) */
    unsigned int* const symbol_value_out ); /* On success this is set to the value associated with symbol_name.
                                               (typically the address of a static variable given by symbol_name) */

/* Description: Provides the number of boards in the current system. Note that this does not include simulators. It is still
                necessary to call CSAPI_new before this function to load the library of functions. It is valid to
                create a CSAPIState with CSAPI_new, call this function and then delete the state with CSAPI_delete.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_arg */
DRVErrno CSAPI_num_cards(
    struct CSAPIState* const s,    /* State created by CSAPI_new. Required to ensure the dynamic CSAPI library is loaded */
    unsigned int* const num_of_cards ); /* Pointer to value to be given the number of boards in the current system */

/* Description: Provides the number of processors on the board we are connected to.
   Returns: DRVErrno_success,  DRVErrno_bad_csapi_state, DRVErrno_bad_csapi_arg */
DRVErrno CSAPI_num_processors(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int* const num_of_processors ); /* Pointer to value to be given the number of processors */

/* Description: Provides the number of processing elements on the specified processor on the board we are connected to.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_state, DRVErrno_invalid_proc_inx, DRVErrno_bad_csapi_arg */
DRVErrno CSAPI_num_pes(
    struct CSAPIState* const s,    /* State created by CSAPI_new */
    unsigned int proc_inx,         /* Index of processor to get count of processing elements on. CS_PROCESSOR0 or CS_PROCESSOR1 */
    unsigned int* const num_of_pes ); /* Pointer to value to be given the number of processing elements */

/* Description: This function decodes an error_code and copies a short description of the error to the provided char buffer.
                The string will be truncated at max_string_length if necessary to avoid over-running the provided buffer.
   Returns: DRVErrno_success, DRVErrno_bad_csapi_arg */
DRVErrno CSAPI_get_error_string(
    struct CSAPIState* const s,    /* State created by CSAPI_new. Required to ensure the dynamic CSAPI library is loaded */
    const DRVErrno error_code,     /* Error code to get the string for. Must be of type DRVErrno */
    char* const error_string,      /* Pointer to pre-allocated buffer to receive error string */
    const int max_string_length ); /* Length of pre-allocated buffer to receive error string */


#ifdef __cplusplus
}
#endif
    
#endif /* _CLEARD_INTERFACE_CSAPI_INCLUDE_CSAPI_H_ */

