//
//
//      Copyright (c) ClearSpeed Technology plc 2000,2002,2006
//      All rights reserved.
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. Any use is at your own risk. Permission to use or copy this
// software for any purpose is hereby granted without fee, provided the
// above notices are retained on all copies. Permission to modify the code
// and to distribute modified code is granted, provided the above notices
// are retained, and a notice that the code was modified is included with
// the above copyright notice.
//
//


// Original creation
// Author   : Michal Harasimiuk
// Date     : 28-feb-2001
//
// endianness conversion routines

#ifndef _COMMON_GENERIC_ENDIAN_H_
#define _COMMON_GENERIC_ENDIAN_H_

#ifdef __cplusplus
  namespace Generic {
# define INLINE inline
# define TAG
#else
# ifdef _WIN32
#  define INLINE _inline 
# else
#  define INLINE inline extern
# endif
#define TAG enum
#endif

#ifdef EMIT_BODIES
# undef  INLINE
# define INLINE 
#endif


#include <assert.h>


/*!
  \brief Indicate whether we are using a big-endian or little-endian
  value.

*/
enum Endianness
{
 EN_Unknown = 0,
 EN_LittleEndian,
 EN_BigEndian,

 //-------------
 
 EN__NumTypes,
 EN_Invalid = EN_Unknown
};

#ifdef __cplusplus
INLINE const char* format( TAG Endianness e )
#else
INLINE const char* Endianness_format( TAG Endianness e )
#endif 
{
    char* res = "-- unknown --";
    if     ( e == EN_LittleEndian ) res = "little";
    else if( e == EN_BigEndian    ) res = "big"   ;

    return res;
}

/*!
  translate from native to lsb
*/
INLINE unsigned int int2lsb( unsigned int val )
{
    unsigned int mem;
    unsigned char *r = (unsigned char *)&mem;  // Make sure mem alignment ok for solaris

    r[0] = (0xff & val);
    r[1] = (0xff & (val>>8));
    r[2] = (0xff & (val>>16));
    r[3] = (0xff & (val>>24));

//    return *((unsigned int*)r);
    return mem;
} 

/*!
  translate from native to msb
*/
INLINE unsigned int int2msb( unsigned int val )
{
    unsigned int mem;
    unsigned char *r = (unsigned char *)&mem; // Make sure mem alignment ok for solaris

    r[0] = (0xff & (val>>24));
    r[1] = (0xff & (val>>16));
    r[2] = (0xff & (val>>8));
    r[3] = (0xff & val);

//    return *((unsigned int*)r);
    return mem;
}

/*!
  translate from native to lsb
*/
INLINE unsigned short short2lsb( unsigned short val )
{
    unsigned short mem; 
    unsigned char *r = (unsigned char *)&mem; // Make sure mem alignment ok for solaris

    r[0] = (0xff & val);
    r[1] = (0xff & (val>>8));

//    return *((unsigned short*)r);
    return mem;
} 

/*!
  translate from native to msb
*/
INLINE unsigned short short2msb( unsigned short val )
{
    unsigned short mem;
    unsigned char *r = (unsigned char *)&mem; // Make sure mem alignment ok for solaris

    r[0] = (0xff & (val>>8));
    r[1] = (0xff & val);

//    return *((unsigned short*)r);
    return mem;
}

/*!
  translate from lsb to native
*/
INLINE unsigned int lsb2int( unsigned int val )
{
    unsigned int result = 0;

    union {
        unsigned char r[4];
        unsigned int  lsb;
    } input;


    input.lsb = val;

    result |= ( ((unsigned int)input.r[0])       );
    result |= ( ((unsigned int)input.r[1]) <<  8 );
    result |= ( ((unsigned int)input.r[2]) << 16 );
    result |= ( ((unsigned int)input.r[3]) << 24 );


 return result;
} 

/*!
  translate from msb to native
*/
INLINE unsigned int msb2int( unsigned int val )
{
    unsigned int result = 0;

    union {
        unsigned char r[4];
        unsigned int  msb;
    } input;


    input.msb = val;

    result |= ( ((unsigned int)input.r[3])       );
    result |= ( ((unsigned int)input.r[2]) <<  8 );
    result |= ( ((unsigned int)input.r[1]) << 16 );
    result |= ( ((unsigned int)input.r[0]) << 24 );


 return result;
}

/*!
  translate from lsb to native
*/
INLINE unsigned short lsb2short( unsigned short val )
{
    unsigned short result = 0;

    union {
        unsigned char  r[2];
        unsigned short lsb;
    } input;


    input.lsb = val;

    result |= ( input.r[0]       );
    result |= ( input.r[1] <<  8 );



 return result;
} 

/*!
  translate from msb to native
*/
INLINE unsigned short msb2short( unsigned short val )
{
    unsigned short result = 0;

    union {
        unsigned char  r[2];
        unsigned short msb;
    } input;


    input.msb = val;

    result |= ( input.r[1]       );
    result |= ( input.r[0] <<  8 );



 return result;
}

/*!
  returns the endianness of the machine we are on
*/
INLINE TAG Endianness getNativeEncoding()
{
    unsigned short __encoding = EN_LittleEndian + (EN_BigEndian << 8);

    return (TAG Endianness)(*(unsigned char*)&__encoding);
}


/*!
  converts the in_buffer into the out_buffer<br>
  the endianness parameter reflects the non_native endianness , which is to be understood 
  as the external endianness ( the source buffer if we convert from external to native, 
  or the target buffer, if we go the oposite direction )

  this function works in any direction
*/
INLINE void* bufferConvert( void* out , void* in , TAG Endianness non_native_endian , int size )
{
    TAG Endianness native = getNativeEncoding();

    int ix = 0, ox, os;

    if( native == non_native_endian )   // byte copy, no inversion required
    {
        ox = 0;
        os = 1;
    }
    else                                // reverse byte copy
    {
        ox = size-1;
        os = -1;
    }

    // the copying loop

    while( ix < size )
    {
        *((char*)out + ox ) = *((char*)in + ix );
        
        ix++;
        ox += os;
    }

    return out;
}

/*!
  this function converts the incomming buffer to little endian
  the input_endian parameter tells the endianness of the in buffer <br><br>

  the function is usefull, because the little endian is easier to work with in 
  certain circumstances - imagine, we have an short which has been sign extended 
  to an int and we want to perform some masking ops on it, but we want to treat 
  the buffer as a 2 byte entity. Native encoding provides us with no way of 
  knowing where these relevant bytes are. In LE, we know where they are

  this function can also be used for conversion in the other direction, 
  when the input buffer is LE and the output must be in the given endianness

*/
INLINE void* bufferLSB( void* out , void* in , TAG Endianness endian , int size )
{
    int ix = 0, ox, os;             // in index, out index, step

    assert( size > 0 );

    if( size > 0 )
    {

        if( endian == EN_LittleEndian ) // byte copy, no inversion required
        {
            ox = 0;
            os = 1;
        }
        else                                // reverse byte copy
        {
            ox = size-1;
            os = -1;
        }

        // the copying loop

        while( ix < size )
        {
            *((char*)out + ox ) = *((char*)in + ix );
        
            ix++;
            ox += os;
        }
    }

    return out;
}



/*!
  sign extends the number given via the void* buffer from size 'from' to size 'to'
  it is assumed that the buffer is large enough to support such operation, 
  no checks are made <br><br>

  it works only for numbers already expressed in the native format

  to sign extend from char to short, from=1, to=2
*/
INLINE void* bufferSignExtend( void* buffer , int from , int to )
{
    unsigned char mask = 0;

    //assert( to - from >= 0 );

    TAG Endianness native = getNativeEncoding();

    if( native == EN_BigEndian )
    {
        int size_diff = to - from;

        // here, the bytes ordering in storage for the number 0x11223344 will be:
        // [0]=0x11 [1]=0x22 [2]=0x33 [3]=0x44
        //
        // we therefore have to move the bytes first and sign extend from 0 into 
        // growing indices

        // get the mask first

        mask = (((unsigned char*)buffer)[0] & 0x80)?0xFF:0x00;

        // shift

        for( from--,to-- ; from>=0 ; from--,to-- ) 
        {
            ((unsigned char*)buffer)[to] = ((unsigned char*)buffer)[from];
        }

        // extend 

        for( size_diff-- ; size_diff>=0 ; size_diff-- )
        {
            ((unsigned char*)buffer)[size_diff] = mask;
        }

    }
    else
    {
        // here, the bytes ordering in storage for the number 0x11223344 will be:
        // [0]=0x44 [1]=0x33 [2]=0x22 [3]=0x11
        //
        // we are therefore sign extending into the growing indices

        // get the mask first

        mask = (((unsigned char*)buffer)[from-1] & 0x80)?0xFF:0x00;

        // extend

        for( ; from < to ; from++ ) ((unsigned char*)buffer)[from] = mask;
    }

    return buffer;
}

#ifdef __cplusplus
}
#endif

#endif /* _COMMON_GENERIC_ENDIAN_H_ */

